
# THIS SCRIPT IS LICENSED UNDER GPL,
# please read the license block.
#
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

bl_info = {
	"name": "Logic to States",
	"author": "",
	"version": (1, 0),
	"blender": (2, 76, 0),
	"location": "Logic Editor > Properties > Logic to States",
	"description": "Copy logic bricks from state to state. If sensors / actuators are connected to more than one controller, it will duplicate those.",
	"warning": "",
	"wiki_url": "",
	"category": "Game Engine",
} 



import bpy

from bpy.props import IntProperty
from bpy.types import Menu, Panel


class CopyLogicStateToStatePanel(bpy.types.Panel):
	"""Creates a Panel in the Logic Editor properties window""" 
	bl_label = "Logic to States"
	bl_options = {'DEFAULT_CLOSED'} 
	bl_idname = "OBJECT_copy_logic_setup"
	bl_space_type = 'LOGIC_EDITOR'
	bl_region_type = 'UI'



	def draw(self, context):
		layout = self.layout

		act_obj = context.active_object		 

		row = layout.row(align=True)
		row.scale_x = .65				 
		row.prop(act_obj, "copy_to_state", text= 'To')
		row.prop(act_obj, "copy_from_state", text= 'From')
		row = layout.row(align=True)
		row.scale_x = .8
		row.operator("object.copy_state_to_state", text= 'Copy to State')		
################################################################################################


class LogicStateToState(bpy.types.Operator):
	"""Copy logic from the current state to the selected state"""  
	bl_idname = "object.copy_state_to_state"
	bl_label = "Copy to States"

	
	def execute(self, context):


		object = bpy.context.active_object
		select_active_state = context.object.copy_from_state
		state_change = context.object.copy_to_state
		change_name = "." + "%s" % state_change


		################################################################################################
		sensorsByController = {}
		for sensor in object.game.sensors:
			for controller in sensor.controllers:
				sensors = sensorsByController.get(controller, [])
				sensorsByController[controller] = sensors
				sensors.append(sensor)




		for controller in list(object.game.controllers):
			
			
			if controller.states == select_active_state:    	
				bpy.ops.logic.controller_add(type = controller.type)
				newController = object.game.controllers[-1]

			################################################################################################
			#Start of LOGIC_AND Controller
			#If + in controller name and controller type is == to LOGIC_AND, add a new LOGIC_AND controller and copy settings   
				if controller.type == 'LOGIC_AND': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active 

			#End of LOGIC_AND Controller
			################################################################################################			


			################################################################################################
			#Start of LOGIC_OR Controller 
			#If + in controller name and controller type is == to LOGIC_OR, add a new LOGIC_OR controller and copy settings 
				if controller.type == 'LOGIC_OR':  
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active

			#End of LOGIC_OR Controller
			################################################################################################

				
			################################################################################################
			#Start of LOGIC_NAND Controller
			#If + in controller name and controller type is == to LOGIC_NAND, add a new LOGIC_NAND controller and copy settings 
				if controller.type == 'LOGIC_NAND': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active

			#End of LOGIC_NAND Controller
			################################################################################################


			################################################################################################
			#Start of LOGIC_NOR Controller 
			#If + in controller name and controller type is == to LOGIC_NOR, add a new LOGIC_NOR controller and copy settings   
				if controller.type == 'LOGIC_NOR': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active

			#End of LOGIC_NOR Controller
			################################################################################################


			################################################################################################
			#Start of LOGIC_XOR Controller
			#If + in controller name and controller type is == to LOGIC_XOR, add a new LOGIC_XOR controller and copy settings   
				if controller.type == 'LOGIC_XOR': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active

			#End of LOGIC_XOR Controller
			################################################################################################


			################################################################################################
			#Start of LOGIC_XNOR Controller 
			#If + in controller name and controller type is == to LOGIC_XNOR, add a new LOGIC_XNOR controller and copy settings 
				if controller.type == 'LOGIC_XNOR': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active

			#End of LOGIC_XNOR Controller
			################################################################################################


			################################################################################################
			#Start of EXPRESSION Controller
			#If + in controller name and controller type is == to EXPRESSION, add a new EXPRESSION controller and copy settings 
				if controller.type == 'EXPRESSION': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active
					newController.expression = controller.expression 

			#End of EXPRESSION Controller
			################################################################################################


			################################################################################################
			#Start of PYTHON Controller 
			#If + in controller name and controller type is == to PYTHON, add a new PYTHON controller and copy settings 
				if controller.type == 'PYTHON': 
					newController.show_expanded = controller.show_expanded  
					newController.name = controller.name + "%s" % change_name
					newController.use_priority = controller.use_priority
					newController.active = controller.active
					newController.mode = controller.mode

					if controller.mode == 'MODULE':
						newController.module = controller.module
						newController.use_debug = controller.use_debug

					if controller.mode == 'SCRIPT':
						newController.text = controller.text   

			#End of PYTHON Controller
			################################################################################################
			
				#Move copied controller to new state			
				newController.states = state_change
			#End of Copying Controllers
			################################################################################################	
				
				for sensor in sensorsByController[controller]:
					bpy.ops.logic.sensor_add(type = sensor.type)
					newSensor = object.game.sensors[-1]		
					newController.link(sensor=newSensor)

			################################################################################################
			#Start of Always Sensor
			#If + in sensor name and sensor type is == to ALWAYS, add new ALWAYS sensors and copy settings  
					if sensor.type == 'ALWAYS':   
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
			#End of Always Sensor
			################################################################################################


			################################################################################################
			#Start of Armature Sensor   
			#If + in sensor name and sensor type is == to ARMATURE, add new ARMATURE sensors and copy settings  
					if sensor.type== 'ARMATURE':   
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.bone = sensor.bone
						newSensor.test_type = sensor.test_type
						
						if sensor.test_type != 'STATECHG':
							newSensor.value = sensor.value
			#End of Armature Sensor
			################################################################################################


			################################################################################################
			#Start of Actuator Sensor   
			#If + in sensor name and sensor type is == to ACTUATOR, add a new ACTUATOR sensor and copy settings  
					if sensor.type == 'ACTUATOR': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.actuator = sensor.actuator		  
			#End of Actuator Sensor
			################################################################################################	


			################################################################################################
			#Start of Collision Sensor  
			#If + in sensor name and sensor type is == to COLLISION, add a new COLLISION sensor and copy settings   
					if sensor.type == 'COLLISION': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.use_pulse = sensor.use_pulse
						newSensor.use_material = sensor.use_material	 
						
						if sensor.use_material == True:
							newSensor.material = sensor.material

						if sensor.use_material == False:
							newSensor.property = sensor.property									 
			#End of Collision Sensor
			################################################################################################	


			################################################################################################
			#Start of Delay Sensor  
			#If + in sensor name and sensor type is == to DELAY, add a new DELAY sensor and copy settings   
					if sensor.type == 'DELAY': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.delay = sensor.delay
						newSensor.duration = sensor.duration
						newSensor.use_repeat = sensor.use_repeat		  
			#End of Delay Sensor
			################################################################################################


			################################################################################################
			#Start of Joystick Sensor
			#If + in sensor name and sensor type is == to JOYSTICK, add a new JOYSTICK sensor and copy settings  
					if sensor.type == 'JOYSTICK': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.joystick_index = sensor.joystick_index
						newSensor.event_type = sensor.event_type
						newSensor.use_all_events = sensor.use_all_events	 
						
						if sensor.event_type == 'BUTTON':
							newSensor.button_number = sensor.button_number  

						if sensor.event_type == 'AXIS':
							newSensor.axis_number = sensor.axis_number
							newSensor.axis_threshold = sensor.axis_threshold
							newSensor.axis_direction = sensor.axis_direction 

						if sensor.event_type == 'HAT':
							newSensor.hat_number = sensor.hat_number
							newSensor.hat_direction = sensor.hat_direction

						if sensor.event_type == 'AXIS_SINGLE':
							newSensor.single_axis_number = sensor.single_axis_number
							newSensor.axis_threshold = sensor.axis_threshold		  
			#End of Joystick Sensor
			################################################################################################


			################################################################################################
			#Start of Keyboard Sensor
			#If + in sensor name and sensor type is == to KEYBOARD, add a new KEYBOARD sensor and copy settings  
					if sensor.type == 'KEYBOARD': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.key = sensor.key
						newSensor.use_all_keys = sensor.use_all_keys
						newSensor.modifier_key_1 = sensor.modifier_key_1
						newSensor.modifier_key_2 = sensor.modifier_key_2   
						newSensor.log = sensor.log
						newSensor.target = sensor.target		  
			#End of Keyboard Sensor
			################################################################################################


			################################################################################################
			#Start of Message Sensor
			#If + in sensor name and sensor type is == to MESSAGE, add a new MESSAGE sensor and copy settings   
					if sensor.type == 'MESSAGE':  
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.subject = sensor.subject  
			#End of Message Sensor
			################################################################################################


			################################################################################################
			#Start of Mouse Sensor
			#If + in sensor name and sensor type is == to MOUSE, add a new MOUSE sensor and copy settings   
					if sensor.type == 'MOUSE': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.mouse_event = sensor.mouse_event
						
						if sensor.mouse_event == 'MOUSEOVERANY':
							newSensor.use_pulse = sensor.use_pulse
							newSensor.use_material = sensor.use_material  
							newSensor.use_x_ray = sensor.use_x_ray

							if sensor.use_material == 'MATERIAL':
								newSensor.material = sensor.material

							if sensor.use_material == 'PROPERTY':
								newSensor.property = sensor.property		
			#End of Mouse Sensor
			################################################################################################


			################################################################################################
			#Start of Near Sensor
			#If + in sensor name and sensor type is == to NEAR, add a new NEAR sensor and copy settings  
					if sensor.type == 'NEAR': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.property = sensor.property
						newSensor.distance = sensor.distance
						newSensor.reset_distance = sensor.reset_distance	   
			#End of Near Sensor
			################################################################################################


			################################################################################################
			#Start of Property Sensor
			#If + in sensor name and sensor type is == to PROPERTY, add a new PROPERTY sensor and copy settings  
					if sensor.type == 'PROPERTY': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.evaluation_type = sensor.evaluation_type
						newSensor.property = sensor.property
						newSensor.value = sensor.value   
			#End of Property Sensor
			################################################################################################


			################################################################################################
			#Start of Radar Sensor
			#If + in sensor name and sensor type is == to RADAR, add a new RADAR sensor and copy settings   
					if sensor.type == 'RADAR': 
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.property = sensor.property
						newSensor.axis = sensor.axis   
						newSensor.angle = sensor.angle  
						newSensor.distance = sensor.distance	
			#End of Radar Sensor
			################################################################################################


			################################################################################################
			#Start of Random Sensor
			#If + in sensor name and sensor type is == to RANDOM, add a new RANDOM sensor and copy settings  
					if sensor.type == 'RANDOM':   
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.seed = sensor.seed 
			#End of Random Sensor
			################################################################################################


			################################################################################################
			#Start of Ray Sensor
			#If + in sensor name and sensor type is == to RAY, add a new RAY sensor and copy settings   
					if sensor.type == 'RAY':  
						newSensor.show_expanded = sensor.show_expanded  
						newSensor.name = sensor.name + "%s" % change_name   
						newSensor.pin = sensor.pin
						newSensor.active = sensor.active
						newSensor.use_pulse_true_level = sensor.use_pulse_true_level
						newSensor.use_pulse_false_level = sensor.use_pulse_false_level  
						newSensor.tick_skip = sensor.tick_skip
						newSensor.use_level = sensor.use_level   
						newSensor.use_tap = sensor.use_tap
						newSensor.invert = sensor.invert
						newSensor.ray_type = sensor.ray_type
						newSensor.axis = sensor.axis
						newSensor.range = sensor.range 
						newSensor.use_x_ray = sensor.use_x_ray

						if sensor.ray_type == 'MATERIAL':
							newSensor.material = sensor.material

						if sensor.ray_type == 'PROPERTY':
							newSensor.property = sensor.property	
			#End of Ray Sensor
			################################################################################################
			#End of Copying Sensors
			################################################################################################
				
				for actuator in controller.actuators:
					bpy.ops.logic.actuator_add(type = actuator.type)
					newActuator = object.game.actuators[-1]		
					newController.link(actuator=newActuator)

			################################################################################################
			#Start of Armature Actuator
			#If + in actuator name and actuator type is == to ARMATURE, add new ARMATURE actuators and copy settings  
					if actuator.type == 'ARMATURE':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'ENABLE':
							newActuator.bone = actuator.bone 

						if actuator.mode == 'DISABLE':
							newActuator.bone = actuator.bone

						if actuator.mode == 'SETTARGET':
							newActuator.bone = actuator.bone
							newActuator.target = actuator.target
						
						if actuator.mode == 'SETWEIGHT':
							newActuator.bone = actuator.bone 
							newActuator.weight = actuator.weight	 
						
						if actuator.mode == 'SETINFLUENCE':
							newActuator.bone = actuator.bone	
							newActuator.influence = actuator.influence
			#End of Armature Actuator
			################################################################################################


			################################################################################################
			#Start of Action Actuator
			#If + in actuator name and actuator type is == to ACTION, add new ACTION actuators and copy settings  
					if actuator.type == 'ACTION':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.play_mode = actuator.play_mode
						newActuator.use_force = actuator.use_force
						newActuator.use_additive = actuator.use_additive						
						newActuator.use_local = actuator.use_local
						newActuator.action = actuator.action
						newActuator.use_continue_last_frame = actuator.use_continue_last_frame
						newActuator.frame_start = actuator.frame_start
						newActuator.frame_end = actuator.frame_end   
						newActuator.apply_to_children = actuator.apply_to_children   
						newActuator.frame_blend_in  = actuator.frame_blend_in  
						newActuator.priority = actuator.priority   
						newActuator.layer = actuator.layer
						newActuator.layer_weight = actuator.layer_weight
						newActuator.blend_mode = actuator.blend_mode   
						newActuator.frame_property = actuator.frame_property

						if actuator.play_mode == 'PROPERTY':
							newActuator.property = actuator.property
			#End of Action Actuator
			################################################################################################


			################################################################################################
			#Start of Camera Actuator
			#If + in actuator name and actuator type is == to CAMERA, add new CAMERA actuators and copy settings  
					if actuator.type == 'CAMERA':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.object = actuator.object
						newActuator.height = actuator.height
						newActuator.axis = actuator.axis
						newActuator.min = actuator.min
						newActuator.max = actuator.max
						newActuator.damping = actuator.damping
			#End of Camera Actuator
			################################################################################################


			################################################################################################
			#Start of Constraint Actuator
			#If + in actuator name and actuator type is == to CONSTRAINT, add new CONSTRAINT actuators and copy settings  
					if actuator.type == 'CONSTRAINT':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'LOC':
							newActuator.limit = actuator.limit
							newActuator.limit_min = actuator.limit_min
							newActuator.limit_max = actuator.limit_max
							newActuator.damping = actuator.damping

						if actuator.mode == 'DIST':
							newActuator.direction = actuator.direction
							newActuator.use_local = actuator.use_local
							newActuator.use_normal = actuator.use_normal
							newActuator.use_force_distance = actuator.use_force_distance
							newActuator.range = actuator.range   
							newActuator.distance = actuator.distance
							newActuator.use_material_detect = actuator.use_material_detect   
							newActuator.damping = actuator.damping   
							newActuator.use_persistent = actuator.use_persistent
							newActuator.time = actuator.time   
							newActuator.damping_rotation = actuator.damping_rotation 

							if actuator.use_material_detect == False:
								newActuator.property = actuator.property		  

							if actuator.use_material_detect == True:
								newActuator.material = actuator.material

						if actuator.mode == 'ORI':
							newActuator.direction_axis_pos = actuator.direction_axis_pos
							newActuator.damping = actuator.damping
							newActuator.time = actuator.time   
							newActuator.rotation_max.x = actuator.rotation_max.x
							newActuator.rotation_max.y = actuator.rotation_max.y
							newActuator.rotation_max.z = actuator.rotation_max.z				   
							newActuator.angle_min = actuator.angle_min
							newActuator.angle_max = actuator.angle_max

						if actuator.mode == 'FH':
							newActuator.fh_damping = actuator.fh_damping
							newActuator.fh_height = actuator.fh_height
							newActuator.use_fh_paralel_axis = actuator.use_fh_paralel_axis   
							newActuator.direction_axis = actuator.direction_axis
							newActuator.fh_force = actuator.fh_force
							newActuator.fh_normal = actuator.use_fh_normal
							newActuator.use_material_detect = actuator.use_material_detect  
							newActuator.use_persistent = actuator.use_persistent
							newActuator.time = actuator.time
							newActuator.damping_rotation = actuator.damping_rotation

							if actuator.use_material_detect == False:
								newActuator.property = actuator.property	  

							if actuator.use_material_detect == True:
								newActuator.material = actuator.material	 
			#End of Constraint Actuator
			################################################################################################


			################################################################################################
			#Start of Edit Object Actuator
			#If + in actuator name and actuator type is == to EDIT_OBJECT, add new EDIT_OBJECT actuators and copy settings  
					if actuator.type == 'EDIT_OBJECT':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'ADDOBJECT':
							newActuator.object = actuator.object  
							newActuator.time = actuator.time
							newActuator.linear_velocity.x = actuator.linear_velocity.x
							newActuator.linear_velocity.y = actuator.linear_velocity.y
							newActuator.linear_velocity.z = actuator.linear_velocity.z
							newActuator.use_local_linear_velocity = actuator.use_local_linear_velocity   
							newActuator.angular_velocity.x = actuator.angular_velocity.x   
							newActuator.angular_velocity.y = actuator.angular_velocity.y
							newActuator.angular_velocity.z = actuator.angular_velocity.z		
							newActuator.use_local_angular_velocity = actuator.use_local_angular_velocity

						if actuator.mode == 'REPLACEMESH':
							newActuator.mesh = actuator.mesh	
							newActuator.use_replace_display_mesh = actuator.use_replace_display_mesh
							newActuator.use_replace_physics_mesh = actuator.use_replace_physics_mesh

						if actuator.mode == 'TRACKTO':
							newActuator.track_object = actuator.track_oject  
							newActuator.time = actuator.time
							newActuator.use_3d_tracking = actuator.use_3d_tracking
							newActuator.up_axis = actuator.up_axis
							newActuator.track_axis = actuator.track_axis

						if actuator.mode == 'DYNAMICS':
							newActuator.dynamic_operation = actuator.dynamic_operation

							if actuator.dynamic_operation == 'SETMASS':
								newActuator.mass = actuator.mass
			#End of Edit Object Actuator
			################################################################################################


			################################################################################################
			#Start of Filter 2D Actuator
			#If + in actuator name and actuator type is == to FILTER_2D, add new FILTER_2D actuators and copy settings  
					if actuator.type == 'FILTER_2D':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode
						newActuator.filter_pass = actuator.filter_pass

						if actuator.mode == 'MOTIONBLUR':
							newActuator.motion_blur_factor = motion_blur_factor
							newActuator.use_motion_blur = actuator.use_motion_blur   

						if actuator.mode == 'CUSTOMFILTER':
							newActuator.glsl_shader = actuator.glsl_shader
			#End of Filter 2D Actuator
			################################################################################################


			################################################################################################
			#Start of Game Actuator
			#If + in actuator name and actuator type is == to GAME, add new GAME actuators and copy settings  
					if actuator.type == 'GAME':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'START':
							newActuator.filename = actuator.filename
			#End of Game Actuator
			################################################################################################


			################################################################################################
			#Start of Message Actuator
			#If + in actuator name and actuator type is == to MESSAGE, add new MESSAGE actuators and copy settings  
					if actuator.type == 'MESSAGE':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.to_property = actuator.to_property
						newActuator.subject = actuator.subject
						newActuator.body_type = actuator.body_type

						if actuator.body_type == 'TEXT':
							newActuator.body_message = actuator.body_message

						if actuator.body_type == 'PROPERTY':
							newActuator.body_property = actuator.body_property
			#End of Message Actuator
			################################################################################################


			################################################################################################
			#Start of Mouse Actuator
			#If + in actuator name and actuator type is == to MOUSE, add new MOUSE actuators and copy settings  
					if actuator.type == 'MOUSE':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'VISIBILITY':
							newActuator.visible = actuator.visible

						if actuator.mode == 'LOOK':
							newActuator.use_axis_x = actuator.use_axis_x
							newActuator.sensitivity_x = actuator.sensitivity_x   
							newActuator.threshold_x = actuator.threshold_x   
							newActuator.min_x = actuator.min_x   
							newActuator.max_x = actuator.max_x   
							newActuator.object_axis_x = actuator.object_axis_x
							newActuator.local_x = actuator.local_x   
							newActuator.reset_x = actuator.reset_x  
							newActuator.use_axis_y = actuator.use_axis_y
							newActuator.sensitivity_y = actuator.sensitivity_y   
							newActuator.threshold_y = actuator.threshold_y   
							newActuator.min_y = actuator.min_y   
							newActuator.max_y = actuator.max_y   
							newActuator.object_axis_y = actuator.object_axis_y
							newActuator.local_y = actuator.local_y   
							newActuator.reset_y = actuator.reset_y
			#End of Mouse Actuator
			################################################################################################


			################################################################################################
			#Start of Motion Actuator
			#If + in actuator name and actuator type is == to MOTION, add new MOTION actuators and copy settings  
					if actuator.type == 'MOTION':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode
						physics_type = object.game.physics_type

						if actuator.mode == 'OBJECT_NORMAL':
							newActuator.offset_location.x = actuator.offset_location.x
							newActuator.offset_location.y = actuator.offset_location.y   
							newActuator.offset_location.z = actuator.offset_location.z
							newActuator.use_local_location = actuator.use_local_location   
							newActuator.offset_rotation.x = actuator.offset_rotation.x
							newActuator.offset_rotation.y = actuator.offset_rotation.y   
							newActuator.offset_rotation.z = actuator.offset_rotation.z
							newActuator.use_local_rotation = actuator.use_local_rotation

							if physics_type == 'DYNAMIC':
								newActuator.force.x = actuator.force.x
								newActuator.force.y = actuator.force.y   
								newActuator.force.z = actuator.force.z
								newActuator.use_local_force = actuator.use_local_force   
								newActuator.torque.x = actuator.torque.x
								newActuator.torque.y = actuator.torque.y   
								newActuator.torque.z = actuator.torque.z
								newActuator.use_local_torque = actuator.use_local_torque
								newActuator.linear_velocity.x = actuator.linear_velocity.x
								newActuator.linear_velocity.y = actuator.linear_velocity.y   
								newActuator.linear_velocity.z = actuator.linear_velocity.z
								newActuator.use_local_linear_velocity = actuator.use_local_linear_velocity
								newActuator.use_add_linear_velocity = actuator.use_add_linear_velocity
								newActuator.angular_velocity.x = actuator.angular_velocity.x
								newActuator.angular_velocity.y = actuator.angular_velocity.y   
								newActuator.angular_velocity.z = actuator.angular_velocity.z
								newActuator.use_local_angular_velocity = actuator.use_local_angular_velocity
								newActuator.damping = actuator.damping 

						if actuator.mode == 'OBJECT_SERVO':
							newActuator.reference_object = actuator.reference_object 
							newActuator.linear_velocity.x = actuator.linear_velocity.x
							newActuator.linear_velocity.y = actuator.linear_velocity.y
							newActuator.linear_velocity.z = actuator.linear_velocity.z
							newActuator.use_local_linear_velocity = actuator.use_local_linear_velocity
							newActuator.use_servo_limit_x = actuator.use_servo_limit_x   
							newActuator.force_max_x = actuator.force_max_x
							newActuator.force_min_x = actuator.force_min_x
							newActuator.use_servo_limit_y = actuator.use_servo_limit_y   
							newActuator.force_max_y = actuator.force_max_y
							newActuator.force_min_y = actuator.force_min_y
							newActuator.use_servo_limit_z = actuator.use_servo_limit_z   
							newActuator.force_max_z = actuator.force_max_z
							newActuator.force_min_z = actuator.force_min_z
							newActuator.proportional_coefficient = actuator.proportional_coefficient   
							newActuator.integral_coefficient = actuator.integral_coefficient
							newActuator.derivate_coefficient = actuator.derivate_coefficient

						if actuator.mode == 'OBJECT_CHARACTER':
							newActuator.offset_location.x = actuator.offset_location.x
							newActuator.offset_location.y = actuator.offset_location.y   
							newActuator.offset_location.z = actuator.offset_location.z
							newActuator.use_local_location = actuator.use_local_location
							newActuator.use_add_character_location = actuator.use_add_character_location		
							newActuator.offset_rotation.x = actuator.offset_rotation.x
							newActuator.offset_rotation.y = actuator.offset_rotation.y   
							newActuator.offset_rotation.z = actuator.offset_rotation.z
							newActuator.use_local_rotation = actuator.use_local_rotation
							newActuator.use_character_jump = actuator.use_character_jump
			#End of Motion Actuator
			################################################################################################


			################################################################################################
			#Start of Parent Actuator
			#If + in actuator name and actuator type is == to PARENT, add new PARENT actuators and copy settings  
					if actuator.type == 'PARENT':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'SETPARENT':
							newActuator.object = actuator.object 
							newActuator.use_compound = actuator.use_compound
							newActuator.use_ghost = actuator.use_ghost
			#End of Parent Actuator
			################################################################################################


			################################################################################################
			#Start of Property Actuator
			#If + in actuator name and actuator type is == to PROPERTY, add new PROPERTY actuators and copy settings  
					if actuator.type == 'PROPERTY':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'ASSIGN' or actuator.mode == 'ADD':
							newActuator.property = actuator.property
							newActuator.value = actuator.value

						if actuator.mode == 'TOGGLE' or actuator.mode == 'LEVEL' or actuator.mode == 'COPY':
							newActuator.property = actuator.property
			#End of Property Actuator
			################################################################################################


			################################################################################################
			#Start of Random Actuator
			#If + in actuator name and actuator type is == to RANDOM, add new RANDOM actuators and copy settings  
					if actuator.type == 'RANDOM':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.distribution = actuator.distribution

						if actuator.distribution == 'BOOL_CONSTANT':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.use_always_true = actuator.use_always_true   

						if actuator.distribution == 'BOOL_UNIFORM':  
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property

						if actuator.distribution == 'BOOL_BERNOUILLI':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.chance = actuator.chance

						if actuator.distribution == 'INT_CONSTANT':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.value = actuator.int_value

						if actuator.distribution == 'INT_UNIFORM':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.int_min = actuator.int_min
							newActuator.int_max = actuator.int_max   

						if actuator.distribution == 'INT_POISSON':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.int_mean = actuator.int_mean					  

						if actuator.distribution == 'FLOAT_CONSTANT':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.float_value = actuator.float_value

						if actuator.distribution == 'FLOAT_UNIFORM':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.float_min = actuator.float_min
							newActuator.float_max = actuator.float_max

						if actuator.distribution == 'FLOAT_NORMAL':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.float_mean = actuator.float_mean
							newActuator.standard_derivation = actuator.standard_derivation   

						if actuator.distribution == 'FLOAT_NEGATIVE_EXPONENTIAL':
							newActuator.seed = actuator.seed   
							newActuator.property = actuator.property
							newActuator.half_life_time = actuator.half_life_time
			#End of Random Actuator
			################################################################################################


			################################################################################################
			#Start of Scene Actuator
			#If + in actuator name and actuator type is == to SCENE, add new SCENE actuators and copy settings  
					if actuator.type == 'SCENE':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode

						if actuator.mode == 'CAMERA':
							newActuator.camera = actuator.camera

						if actuator.mode != 'CAMERA' or actuator.mode != 'RESTART':
							newActuator.scene = actuator.scene
			#End of Scene Actuator
			################################################################################################


			################################################################################################
			#Start of Steering Actuator
			#If + in actuator name and actuator type is == to STEERING, add new STEERING actuators and copy settings  
					if actuator.type == 'STEERING':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.mode = actuator.mode
						newActuator.target = actuator.target
						newActuator.navmesh = actuator.navmesh
						newActuator.distance = actuator.distance
						newActuator.velocity = actuator.velocity
						newActuator.acceleration = actuator.acceleration
						newActuator.turn_speed = actuator.turn_speed   
						newActuator.facing = actuator.facing
						newActuator.facing_axis = actuator.facing_axis  
						newActuator.normal_up = actuator.normal_up  
						newActuator.self_terminated = actuator.self_terminated
						newActuator.lock_z_velocity = actuator.lock_z_velocity  
						newActuator.show_visualization = actuator.show_visualization

						if actuator.mode == 'PATHFOLLOWING':
							newActuator.update_period = actuator.update_period 
			#End of Steering Actuator
			################################################################################################


			################################################################################################
			#Start of Sound Actuator
			#If + in actuator name and actuator type is == to SOUND, add new SOUND actuators and copy settings  
					if actuator.type == 'SOUND':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
			#End of Sound Actuator
			################################################################################################


			################################################################################################
			#Start of State Actuator
			#If + in actuator name and actuator type is == to STATE, add new STATE actuators and copy settings  
					if actuator.type == 'STATE':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.operation = actuator.operation
						newActuator.states[0] = actuator.states[0]
						newActuator.states[1] = actuator.states[1]
						newActuator.states[2] = actuator.states[2]
						newActuator.states[3] = actuator.states[3]
						newActuator.states[4] = actuator.states[4]
						newActuator.states[5] = actuator.states[5]
						newActuator.states[6] = actuator.states[6]
						newActuator.states[7] = actuator.states[7]
						newActuator.states[8] = actuator.states[8]
						newActuator.states[9] = actuator.states[9]
						newActuator.states[10] = actuator.states[10]
						newActuator.states[11] = actuator.states[11]
						newActuator.states[12] = actuator.states[12]
						newActuator.states[13] = actuator.states[13]
						newActuator.states[14] = actuator.states[14]
						newActuator.states[15] = actuator.states[15]
						newActuator.states[16] = actuator.states[16]
						newActuator.states[17] = actuator.states[17]
						newActuator.states[18] = actuator.states[18]
						newActuator.states[19] = actuator.states[19]
						newActuator.states[20] = actuator.states[20]
						newActuator.states[21] = actuator.states[21]
						newActuator.states[22] = actuator.states[22]
						newActuator.states[23] = actuator.states[23]
						newActuator.states[24] = actuator.states[24]
						newActuator.states[25] = actuator.states[25]
						newActuator.states[26] = actuator.states[26]
						newActuator.states[27] = actuator.states[27]
						newActuator.states[28] = actuator.states[28]
						newActuator.states[29] = actuator.states[29]
			#End of State Actuator
			################################################################################################


			################################################################################################
			#Start of Visibility Actuator
			#If + in actuator name and actuator type is == to VISIBILITY, add new VISIBILITY actuators and copy settings  
					if actuator.type == 'VISIBILITY':   
						newActuator.show_expanded = actuator.show_expanded   
						newActuator.name = actuator.name + "%s" % change_name
						newActuator.pin = actuator.pin
						newActuator.active = actuator.active
						newActuator.use_visible = actuator.use_visible
						newActuator.use_occlusion = actuator.use_occlusion  
						newActuator.apply_to_children = actuator.apply_to_children
			#End of Visibility Actuator
			################################################################################################
		return {'FINISHED'}
#End of copying logic state to state operator
######################################################################################	
	
	
		
	

################################################################################################
def register():
	bpy.utils.register_class(CopyLogicStateToStatePanel)
	bpy.utils.register_class(LogicStateToState)

	bpy.types.Object.copy_from_state = bpy.props.IntProperty(
		name = "",
		description = "Change the number to the state to copy from(active state).",
		min = 1 ,
		max = 30
	  )
	bpy.types.Object.copy_to_state = bpy.props.IntProperty(
		name = "",
		description = "Change the number to the state to copy to.",
		min = 1 ,
		max = 30
	  )   


	
def unregister():
	bpy.utils.unregister_class(CopyLogicStateToStatePanel)
	bpy.utils.unregister_class(LogicStateToState)

	del bpy.types.Object.copy_from_state
	del bpy.types.Object.copy_to_state
		  
	
if __name__ == "__main__":
	register()

